import { EquipmentSlot, GameMode, world } from "@minecraft/server";

/**
 * @param {number} min The minimum integer
  * @param {number} max The maximum integer
   * @returns {number} A random integer between the `min` and `max` parameters (inclusive)
    */
const randomInt = (min, max) => Math.floor(Math.random() * (max - min + 1)) + min;

const maxGrowth = 2;

world.beforeEvents.worldInitialize.subscribe(ev => {
  ev.blockComponentRegistry.registerCustomComponent("jurassic:cycad_sapling_growth", {
    onRandomTick({ block }) {
      const growthChance = 1 / 3;
      if (Math.random() > growthChance) return;
      const growth = block.permutation.getState("jurassic:growth");
      block.setPermutation(block.permutation.withState("jurassic:growth", growth + 1));
      if (block.permutation.withState("jurassic:growth", 2)) {
        block.setType("jurassic:tall_cycad");
        dimension.runCommandAsync(`setblock  ${block.location.x} ${block.location.y + 1} ${block.location.z} jurassic:tall_cycad_top`);
      }
    },
    onPlayerInteract({ block, dimension, player }) {
      if (!player) return;

      const equippable = player.getComponent("minecraft:equippable");
      if (!equippable) return;

      const mainhand = equippable.getEquipmentSlot(EquipmentSlot.Mainhand);
      if (!mainhand.hasItem() || mainhand.typeId !== "minecraft:bone_meal") return;

      if (player.getGameMode() === GameMode.creative) {
        block.setPermutation(block.permutation.withState("jurassic:growth", 2));
      } else {
        let growth = block.permutation.getState("jurassic:growth");
        growth += randomInt(1, maxGrowth - growth);
        block.setPermutation(block.permutation.withState("jurassic:growth", growth));

        if (mainhand.amount > 1) mainhand.amount--;
        else mainhand.setItem(undefined);
      }
      const effectLocation = block.center();
      dimension.playSound("item.bone_meal.use", effectLocation);
      dimension.spawnParticle("minecraft:crop_growth_emitter", effectLocation);
      if (block.permutation.withState("jurassic:growth", 2)) {
        block.setType("jurassic:tall_cycad");
        dimension.runCommandAsync(`setblock  ${block.location.x} ${block.location.y + 1} ${block.location.z} jurassic:tall_cycad_top`);
      }
    },
  });
});